@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2016-2017 Fox Crypto B.V. <openvpn@fox-it.com>
+ *  Copyright (C) 2016-2018 Fox Crypto B.V. <openvpn@fox-it.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -74,6 +74,8 @@
 #ifndef TLSCRYPT_H
 #define TLSCRYPT_H
 
+#ifdef ENABLE_CRYPTO
+
 #include "buffer.h"
 #include "crypto.h"
 #include "session_id.h"
@@ -140,4 +142,6 @@ bool tls_crypt_unwrap(const struct buffer *src, struct buffer *dst,
 
 /** @} */
 
+#endif /* ENABLE_CRYPTO */
+
 #endif /* TLSCRYPT_H */
